import * as core from '@actions/core';
import { execSync } from 'child_process';
import { readFile, writeFile, rm, mkdir } from 'fs';
import { expect, test } from '@jest/globals'
import rimraf from 'rimraf';
import { existsSync } from 'fs';
import { removeDirectorySync } from '../src/io';

function runContainerScript(imageName: string, scriptToExecute: string): string {
  // Write the script to a temporary file
  const os = require('os');
  const path = require('path');

  const tempFilePath = '/tmp/ci_testing/';
  const tempFileName = 'script.sh';
  const tempFileFullPath = path.join(tempFilePath, tempFileName);
  const containerScriptsPath = path.join(os.tmpdir(), 'scripts');
  const containerScriptFullPath = path.join(containerScriptsPath, tempFileName);
  
  removeDirectorySync(tempFilePath);

  // create the directory
  mkdir(tempFilePath, { recursive: true }, (err) => {
    if (err) {
      console.error(`Failed to create directory: ${tempFilePath}`);
      throw err;
    }

    // write the file to the temp file
    writeFile(tempFileFullPath, scriptToExecute.toString(), (err) => {
      if (err) {
        console.error(`Failed to write to file: ${tempFileFullPath}`);
        throw err;
      }
      // console.log(`File written successfully: ${tempFileFullPath}`);
    });
  });

  // Execute the script inside the container

  // Check if the repo is o3de-extras

  const repoName = execSync(`pwd`).toString();
  // debug print the repo name
  console.log(`repoName: ${repoName}`);
  const folderName = repoName.split('/').pop()?.replace('\n', '');

  console.log(`folderName: ${folderName}`);

  // declare the command
  let command = '';

  if (folderName === 'o3de-extras') {
    console.log('o3de-extras detected');
    // if it is o3de-extras, then we need to mount the workspace
    command = `docker run --rm -v ${tempFilePath}:${containerScriptsPath} -v ${repoName}:/data/workspace/o3de-extras --workdir /data/workspace/o3de-extras ${imageName} /bin/bash ${containerScriptFullPath}`;
  }
  else {
    console.log(`running on a general purpose repo: ${folderName}`);
    command = `docker run --rm -v ${tempFilePath}:${containerScriptsPath} -v ${repoName}:/data/workspace/repository --workdir /data/workspace/repository ${imageName} /bin/bash ${containerScriptFullPath}`;
  }

  // command remove any new line characters
  command = command.replace('\n', '');

  // debug print the command
  console.log(`command: ${command}`);

  const output = execSync(command).toString();

  return output;
}

test('Remove Temp File Test', () => {
  const tempFilePath = '/tmp/ci-test/';

  // Create the directory
  mkdir(tempFilePath, { recursive: true }, (err) => {
    if (err) {
      console.error(`Failed to create directory: ${tempFilePath}`);
      throw err;
    }
  });

  // Remove directory and wait for it to finish
  removeDirectorySync(tempFilePath);

  // Check if the directory exists
  const fileExists = existsSync(tempFilePath);
  expect(fileExists).toBe(false);
});


test('Docker Test', () => {
  let mainOutput = '';
  try {
    // const container = core.getInput('container');
    // const scriptPath = core.getInput('script-path');

    const container = 'khasreto/o3de-extras-daily_dev:latest';
    const scriptPath = 'test/test-script.sh';

    const scriptToExecute = execSync(`cat ${scriptPath}`).toString();

    // Run the main script on the modified container
    mainOutput = runContainerScript(container, scriptToExecute);
    core.info('Main script output:');
    core.info(mainOutput);

    // Perform assertions on the output as needed
  } catch (error) {
    if (error instanceof Error) {
      core.error(error.message);
      core.setFailed(error.message);
    }
  }
  expect(mainOutput).toContain('RESULT: ALL TESTS PASSED');
});